#
# spec file for package perl-Spreadsheet-ReadSXC
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define cpan_name Spreadsheet-ReadSXC
Name:           perl-Spreadsheet-ReadSXC
Version:        0.390.0
Release:        1.4
# 0.39 -> normalize -> 0.390.0
%define cpan_version 0.39
License:        Artistic-1.0 OR GPL-1.0-or-later
Summary:        Extract OpenOffice 1.x spreadsheet data
URL:            https://metacpan.org/release/%{cpan_name}
Source0:        https://cpan.metacpan.org/authors/id/C/CO/CORION/%{cpan_name}-%{cpan_version}.tar.gz
Source1:        cpanspec.yml
# PATCH-FIX-OPENSUSE Do not requires Filter::signatures, as we have perl > 5.20
Patch0:         remove_filter_signatures.diff
BuildArch:      noarch
BuildRequires:  perl
BuildRequires:  perl-macros
BuildRequires:  perl(Archive::Zip) >= 1.34
#BuildRequires:  perl(Filter::signatures) >= 0.160
BuildRequires:  perl(Moo) >= 2
BuildRequires:  perl(PerlIO::gzip)
BuildRequires:  perl(PerlX::Maybe)
BuildRequires:  perl(XML::Twig) >= 3.50
BuildRequires:  perl(XML::Twig::XPath)
BuildRequires:  perl(XML::XPath)
BuildRequires:  perl(XML::XPathEngine)
Requires:       perl(Archive::Zip) >= 1.34
#Requires:       perl(Filter::signatures) >= 0.16
Requires:       perl(Moo) >= 2
Requires:       perl(PerlIO::gzip)
Requires:       perl(PerlX::Maybe)
Requires:       perl(XML::Twig) >= 3.50
Requires:       perl(XML::Twig::XPath)
Requires:       perl(XML::XPath)
Requires:       perl(XML::XPathEngine)
Provides:       perl(Spreadsheet::ParseODS) = %{version}
Provides:       perl(Spreadsheet::ParseODS::Cell) = %{version}
Provides:       perl(Spreadsheet::ParseODS::Settings) = %{version}
Provides:       perl(Spreadsheet::ParseODS::Styles) = %{version}
Provides:       perl(Spreadsheet::ParseODS::Workbook) = %{version}
Provides:       perl(Spreadsheet::ParseODS::Worksheet) = %{version}
Provides:       perl(Spreadsheet::ReadSXC) = %{version}
%undefine       __perllib_provides
%{perl_requires}

%description
Spreadsheet::ReadSXC extracts data from OpenOffice 1.x spreadsheet files
(.sxc). It exports the function read_sxc() which takes a filename and an
optional reference to a hash of options as arguments and returns a
reference to a hash of references to two-dimensional arrays. The hash keys
correspond to the names of worksheets in the OpenOffice workbook. The
two-dimensional arrays correspond to rows and cells in the respective
spreadsheets. If you don't like this because the order of sheets is not
preserved in a hash, read on. The 'OrderBySheet' option provides an array
of hashes instead.

If you prefer to unpack the .sxc file yourself, you can use the function
read_xml_file() instead and pass the path to content.xml as an argument. Or
you can extract the XML string from content.xml and pass the string to the
function read_xml_string(). Both functions also take a reference to a hash
of options as an optional second argument.

Spreadsheet::ReadSXC uses XML::Twig to parse the XML contained in .sxc
files. Only the contents of text:p elements are returned, not the actual
values of table:value attributes. For example, a cell might have a
table:value-type attribute of "currency", a table:value attribute of
"-1500.99" and a table:currency attribute of "USD". The text:p element
would contain "-$1,500.99". This is the string which is returned by the
read_sxc() function, not the value of -1500.99.

Spreadsheet::ReadSXC was written with data import into an SQL database in
mind. Therefore empty spreadsheet cells correspond to undef values in array
rows. The example code above shows how to replace undef values with empty
strings.

If the .sxc file contains an empty spreadsheet its hash element will point
to an empty array (unless you use the 'NoTruncate' option in which case it
will point to an array of an array containing one undefined element).

OpenOffice uses UTF-8 encoding. It depends on your environment how the data
returned by the XML Parser is best handled:

  use Unicode::String qw(latin1 utf8);
  $unicode_string = utf8($$workbook_ref{"Sheet1"}[0][0])->as_string;

  # this will not work for characters outside ISO-8859-1:

  $latin1_string = utf8($$workbook_ref{"Sheet1"}[0][0])->latin1;

Of course there are other modules than Unicode::String on CPAN that handle
conversion between encodings. It's your choice.

Table rows in .sxc files may have a "table:number-rows-repeated" attribute,
which is often used for consecutive empty rows. When you format whole rows
and/or columns in OpenOffice, it sets the numbers of rows in a worksheet to
32,000 and the number of columns to 256, even if only a few lower-numbered
rows and cells actually contain data. Spreadsheet::ReadSXC truncates such
sheets so that there are no empty rows after the last row containing data
and no empty columns after the last column containing data (unless you use
the 'NoTruncate' option).

Still it is perfectly legal for an .sxc file to apply the
"table:number-rows-repeated" attribute to rows that actually contain data
(although I have only been able to produce such files manually, not through
OpenOffice itself). To save on memory usage in these cases,
Spreadsheet::ReadSXC does not copy rows by value, but by reference
(remember that multi-dimensional arrays in Perl are really arrays of
references to arrays). Therefore, if you change a value in one row, it is
possible that you find the corresponding value in the next row changed,
too:

  $$workbook_ref{"Sheet1"}[0][0] = 'new string';
  print $$workbook_ref{"Sheet1"}[1][0];

As of version 0.20 the references returned by read_sxc() et al. remain
valid after subsequent calls to the same function. In earlier versions,
calling read_sxc() with a different file as the argument would change the
data referenced by the original return value, so you had to derefence it
before making another call. Thanks to H. Merijn Brand for fixing this.

%prep
%autosetup  -n %{cpan_name}-%{cpan_version} -p1

find . -type f ! -path "*/t/*" ! -name "*.pl" ! -path "*/bin/*" ! -path "*/script/*" ! -path "*/scripts/*" ! -name "configure" -print0 | xargs -0 chmod 644

%build
perl Makefile.PL INSTALLDIRS=vendor
%make_build

%check
make test

%install
%perl_make_install
%perl_process_packlist
%perl_gen_filelist

%files -f %{name}.files
%doc Changes README
%license LICENSE

%changelog
* Fri Jan 10 2025 Tina Müller <tina.mueller@suse.com>
- Update remove_filter_signatures.diff
* Fri Aug 16 2024 Tina Müller <timueller+perl@suse.de>
- updated to 0.390.0 (0.39)
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.39  2024-08-15
  * Move from Test::More::isn't() to ::isnt()
* Fri Mar  8 2024 Tina Müller <tina.mueller@suse.com>
- Fix disabling of __perllib_provides
* Fri Aug 11 2023 Tina Müller <tina.mueller@suse.com>
- Update remove_filter_signatures.diff
* Sat Jul 22 2023 Tina Müller <timueller+perl@suse.de>
- updated to 0.38
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.38  2023-07-21
  * Various test suite updates, no code changes, no need to upgrade
* Mon Nov 22 2021 Tina Müller <tina.mueller@suse.com>
- Update patch remove_filter_signatures.diff
* Mon Nov 22 2021 Tina Müller <timueller+perl@suse.de>
- updated to 0.37
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.37  2021-11-21
  * Really require XML::Twig 3.50, since that adds the no_xxe option
  0.36  2021-11-21
  * Require XML::Twig 3.50, since that adds the no_xxe option
* Wed Oct 20 2021 Tina Müller <tina.mueller@suse.com>
- Update remove_filter_signatures.diff
* Fri Sep 10 2021 Tina Müller <timueller+perl@suse.de>
- updated to 0.35
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.35  2021-09-08
  * Fix memory leak in the XML parser callbacks
    Reported as GH #8 with a test case, thanks to troost-c!
* Fri Mar 12 2021 Tina Müller <tina.mueller@suse.com>
- Update patch remove_filter_signatures.diff
* Thu Mar 11 2021 Tina Müller <timueller+perl@suse.de>
- updated to 0.34
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.34  2021-03-10
  * Fix OrderBySheet option in Spreadsheet::ReadSXC, thanks to Kuerbis!
* Fri Jan  8 2021 Dirk Stoecker <opensuse@dstoecker.de>
- updated to 0.33
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.33  2020-12-28
  * Read style attributes immediately from Twig. This increases memory usage
    but removes problems when we recycle the Twig object for parsing another
    file
  * Custom styles are now better supported and don't crash the module
    immediately (issue GH #5 , reported with test file by mario-minati)
  * Point bug tracker meta data to Github
* Thu Oct 15 2020 Dirk Stoecker <opensuse@dstoecker.de>
- add remove_filter_signatures.diff patch to drop Filter:signatures requirement
* Wed Oct  7 2020 Tina Müller <timueller+perl@suse.de>
- updated to 0.32
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.32  2020-10-06
  * Eliminate a warning when reading a numeric format without a defined
    number of digits, reported by Tux
  0.31  2020-10-03
  * No code changes, no need to upgrade
  * Change code to work around an ambiguity in Perl to placate Filter::signatures
    / Filter::Simple. "-s _" could also be parsed as "-(s _foo_bar_g)"
    by Filter::Simple.
  0.30  2020-10-02
  * No code changes, no need to upgrade
  * Remove "use XML::Parser;" line, which was not used anymore in the code.
    Spotted by MANWAR
  * This version will be (re)released as 1.00 if the smoke tests come up green
  0.29  2020-09-30
  * Make the test for the error message on non-existing files more lax.
    This addresses RT #133432
  0.28  2020-09-29
  * Require Filter::signatures 0.16 to fix Perl versions below 5.20
  * No upgrade necessary if things already work for you
  0.27  2020-09-28
  * Defang resource intensive test
  * Eliminate "undef" warning when trying to get the active sheet of a
    workbook that does not define an active sheet (GH #3)
  0.26  2020-09-28
  * This needs minimum Perl version 5.010
  * When reading a sheet in readonly mode, more aggressively reuse cell
    objects for repeated cells
  * Construct a cell once, and if it is repeated, clone it if not in readonly
    mode.
    These two changes speed up the parsing of some sheets with many repeated
    cells vastly.
  * Properly handle ODS files without a settings.xml (GH#2)
  0.25  2020-09-27
    This release has major changes and will be promoted to 1.0 if there are
    no major breaks.
  * Added (proper) parsing of ODS files
  * Added support for formats
  * Added support for hidden cells, merged cells
  * Split up parsing and classes into separate files
  * Spreadsheet::ReadSXC now uses Spreadsheet::ParseODS
  * Added ->is_sheet_hidden(), ->get_print_areas, ->get_hyperlink()
  - >is_col_hidden(), ->tab_color(), ->table_styles(), ->filename(),
* Wed Oct  9 2019 <timueller+perl@suse.de>
- updated to 0.24
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
  0.24  2019-10-08
  - Fix parsing the text of cells that only contain "0"
    These were returned as undef, instead of 0 unless the StandardCurrency
    option was used.
  - Fix parsing multiple sheets. If the first cell of the new table was empty,
    the last defined value of the old table was used instead, if the
    real (instead of formatted) values were used.
* Fri Oct  4 2019 <timueller+perl@suse.de>
- updated to 0.23
  see /usr/share/doc/packages/perl-Spreadsheet-ReadSXC/Changes
* Thu Sep  1 2011 chris@computersalat.de
- initial package 0.20
  * created by cpanspec 1.78.04
